pub static APP_ID: &str = "net.ffkkinos.Wildcard";
pub static PROFILE: &str = "Default";
pub static VERSION: &str = "0.3.5";
pub static GETTEXT_PACKAGE: &str = "wildcard";
pub static LOCALEDIR: &str = "/usr/share/locale";
pub static PKGDATADIR: &str = "/usr/share/wildcard";
pub static RESOURCES_FILE: &str = concat!("/usr/share/wildcard", "/resources.gresource");
